use sea_orm::{sea_query::{Alias, Expr, ValueType}, DatabaseConnection, DbErr, *};
use orm::{prelude::*, *, sea_orm_active_enums::Status};

pub async fn get_locations(
    db: &DatabaseConnection
) -> Result<Vec<(locations::Model, Vec<user_tokens::Model>)>, DbErr> {
    Locations::find().find_with_related(UserTokens).all(db).await
}

pub async fn set_chat_id_by_token(
    db: &DatabaseConnection, chat_id: i64, token: &str
) -> Result<(), DbErr> {
    let user_token = UserTokens::find().filter(
        user_tokens::Column::Token.eq(token)
    ).one(db).await?;

    let mut active_user_token: user_tokens::ActiveModel = match user_token {
        Some(res) => res.to_owned().into(),
        None => return Err(DbErr::RecordNotFound(String::new()))
    };

    active_user_token.chat_id = Set(Some(chat_id));

    active_user_token.update(db).await?;

    Ok(())
}

pub async fn set_status_for_location(
    db: &DatabaseConnection, status: Status, location_id: i32
) -> Result<(), DbErr> {

    Locations::update_many()
        .col_expr(locations::Column::Status,
            Expr::value(status).cast_as(Alias::new("status"))
        )
        .filter(locations::Column::Id.eq(location_id))
        .exec(db)
        .await?;

    Ok(())
}